package io.tswf.minecraft.blockventure.core.transmutation.transmute

import groovy.transform.CompileStatic
import io.tswf.minecraft.blockventure.core.api.world.chunk.PersistedChunkCoord
import io.tswf.minecraft.blockventure.core.world.chunk.ChunkPersistService
import io.tswf.transmutations.common.annotation.ClassReference
import io.tswf.transmutations.common.annotation.FieldDescription
import io.tswf.transmutations.common.annotation.MethodDescription
import io.tswf.transmutations.transmute.annotation.AtHead
import io.tswf.transmutations.transmute.annotation.FieldValue
import io.tswf.transmutations.transmute.annotation.ReturningVoidIfTrue
import io.tswf.transmutations.transmute.annotation.Transmute
import io.tswf.transmutations.transmute.meta.MissingTargetStrategy
import it.unimi.dsi.fastutil.longs.Long2ObjectLinkedOpenHashMap
import net.minecraft.client.world.ClientChunkManager
import net.minecraft.client.world.ClientWorld
import net.minecraft.server.world.ChunkHolder
import net.minecraft.server.world.ServerWorld
import net.minecraft.server.world.ThreadedAnvilChunkStorage

import java.util.function.BooleanSupplier

@CompileStatic
class TransmutationThreadedAnvilChunkStorage {

    @Transmute(
            targetClass = @ClassReference(ThreadedAnvilChunkStorage),
            targetMethod = @MethodDescription(
                    name = "unloadChunks",
                    parameterTypeClasses = [ BooleanSupplier ],
                    returnTypeClass = ClassReference.VOID
            ),
            onMissing = MissingTargetStrategy.THROW
    )
    @AtHead
    static void removePersistedChunksFromUnloadQueue(

            BooleanSupplier shouldKeepTicking,

            @FieldValue(@FieldDescription(name = "chunksToUnload", type = @ClassReference(Long2ObjectLinkedOpenHashMap)))
            Long2ObjectLinkedOpenHashMap<ChunkHolder> chunksToUnload,

            @FieldValue(@FieldDescription(name = "world", type = @ClassReference(ServerWorld)))
            ServerWorld world
    ) {
        def chunkCoord = new PersistedChunkCoord()
        chunksToUnload.removeAll {
            chunkCoord.x = it.value.pos.x
            chunkCoord.z = it.value.pos.z

            return ChunkPersistService.isChunkPersisted(world.dimensionKey.value, chunkCoord)
        }
    }

}

@CompileStatic
class TransmutationClientChunkManager {

    @Transmute(
            targetClass = @ClassReference(ClientChunkManager),
            targetMethod = @MethodDescription(
                    name = "unload",
                    parameterTypeClasses = [ ClassReference.INT, ClassReference.INT ],
                    returnTypeClass = ClassReference.VOID
            ),
            onMissing = MissingTargetStrategy.THROW
    )
    @AtHead
    @ReturningVoidIfTrue
    static boolean cancelUnloadClientChunk(

            int chunkX,
            int chunkY,

            @FieldValue(@FieldDescription(name = "world", type = @ClassReference(ClientWorld)))
            ClientWorld clientWorld
    ) {
        return ChunkPersistService.isChunkPersisted(clientWorld.dimensionKey.value, new PersistedChunkCoord(x: chunkX, z: chunkY))
    }
}
