package io.tswf.minecraft.blockventure.core.world.chunk

import groovy.transform.CompileStatic
import io.tswf.minecraft.blockventure.core.api.world.chunk.PersistedChunkCoord
import net.minecraft.util.Identifier

import java.util.concurrent.ConcurrentHashMap
import java.util.concurrent.atomic.AtomicInteger

@CompileStatic
class ChunkPersistService {

    static Map<String, Map<PersistedChunkCoord, AtomicInteger>> persistedChunks = new ConcurrentHashMap<>()

    static void persistChunk(Identifier identifier, PersistedChunkCoord pos) {
        getBlockPersistCounter(identifier, pos).incrementAndGet()
    }

    static void freeChunk(Identifier identifier, PersistedChunkCoord pos) {
        getBlockPersistCounter(identifier, pos).decrementAndGet()
    }

    static boolean isChunkPersisted(Identifier identifier, PersistedChunkCoord pos) {
        return persistedChunks[identifier.toString()]?[pos]?.with { get() > 0 } ?: false
    }

    static AtomicInteger getBlockPersistCounter(Identifier identifier, PersistedChunkCoord pos) {
        def posToCounterMap = persistedChunks.computeIfAbsent(identifier.toString()) {
            new ConcurrentHashMap<>()
        }

        return posToCounterMap.computeIfAbsent(pos) {
            new AtomicInteger()
        }
    }
}
