package io.tswf.minecraft.blockventure.core.world.chunk

import groovy.transform.CompileStatic
import io.tswf.groovy.bettergroovy.transforms.validation.checkedmapconstructor.CheckedMapConstructor
import io.tswf.minecraft.blockventure.core.api.world.chunk.PersistedChunkCoord
import net.minecraft.util.Identifier

import java.util.concurrent.ConcurrentHashMap

@CompileStatic
class ChunkPersistTrackerService {
    static Map<UUID, PersistedChunkTracker> trackers = new ConcurrentHashMap<>()

    static freeChunks(UUID trackerId) {
        trackers[trackerId]?.let {
            for (def chunkPos : it.persistedChunks) {
                ChunkPersistService.freeChunk(it.dimensionKey, chunkPos)
            }
        }

        trackers.remove(trackerId)
    }

    static UUID persistChunks(Identifier dimensionKey, List<PersistedChunkCoord> chunkPositions) {
        def tracker = new PersistedChunkTracker(
                dimensionKey: dimensionKey,
                persistedChunks: chunkPositions
        )

        def trackerId = UUID.randomUUID()

        for (def chunkPos : chunkPositions) {
            ChunkPersistService.persistChunk(dimensionKey, chunkPos)
        }

        trackers[trackerId] = tracker

        return trackerId
    }
}

@CheckedMapConstructor
@CompileStatic
class PersistedChunkTracker {
    Identifier dimensionKey
    List<PersistedChunkCoord> persistedChunks
}
